@@ -358,7 +358,7 @@ var (
 		BottomLeft:       "+",
 		BottomRight:      "+",
 		BottomSeparator:  "+",
-		EmptySeparator:   text.RepeatAndTrim(" ", text.RuneCount("+")),
+		EmptySeparator:   text.RepeatAndTrim(" ", text.RuneWidthWithoutEscSequences("+")),
 		Left:             "|",
 		LeftSeparator:    "+",
 		MiddleHorizontal: "-",
@@ -389,7 +389,7 @@ var (
 		BottomLeft:       "┗",
 		BottomRight:      "┛",
 		BottomSeparator:  "┻",
-		EmptySeparator:   text.RepeatAndTrim(" ", text.RuneCount("╋")),
+		EmptySeparator:   text.RepeatAndTrim(" ", text.RuneWidthWithoutEscSequences("╋")),
 		Left:             "┃",
 		LeftSeparator:    "┣",
 		MiddleHorizontal: "━",
@@ -420,7 +420,7 @@ var (
 		BottomLeft:       "╚",
 		BottomRight:      "╝",
 		BottomSeparator:  "╩",
-		EmptySeparator:   text.RepeatAndTrim(" ", text.RuneCount("╬")),
+		EmptySeparator:   text.RepeatAndTrim(" ", text.RuneWidthWithoutEscSequences("╬")),
 		Left:             "║",
 		LeftSeparator:    "╠",
 		MiddleHorizontal: "═",
@@ -451,7 +451,7 @@ var (
 		BottomLeft:       "└",
 		BottomRight:      "┘",
 		BottomSeparator:  "┴",
-		EmptySeparator:   text.RepeatAndTrim(" ", text.RuneCount("┼")),
+		EmptySeparator:   text.RepeatAndTrim(" ", text.RuneWidthWithoutEscSequences("┼")),
 		Left:             "│",
 		LeftSeparator:    "├",
 		MiddleHorizontal: "─",
@@ -482,7 +482,7 @@ var (
 		BottomLeft:       "╰",
 		BottomRight:      "╯",
 		BottomSeparator:  "┴",
-		EmptySeparator:   text.RepeatAndTrim(" ", text.RuneCount("┼")),
+		EmptySeparator:   text.RepeatAndTrim(" ", text.RuneWidthWithoutEscSequences("┼")),
 		Left:             "│",
 		LeftSeparator:    "├",
 		MiddleHorizontal: "─",
@@ -513,7 +513,7 @@ var (
 		BottomLeft:       "\\",
 		BottomRight:      "/",
 		BottomSeparator:  "v",
-		EmptySeparator:   text.RepeatAndTrim(" ", text.RuneCount("+")),
+		EmptySeparator:   text.RepeatAndTrim(" ", text.RuneWidthWithoutEscSequences("+")),
 		Left:             "[",
 		LeftSeparator:    "{",
 		MiddleHorizontal: "--",
@@ -667,9 +667,10 @@ var (
 
 // FormatOptions defines the text-formatting to perform on parts of the Table.
 type FormatOptions struct {
-	Footer text.Format // footer row(s) text format
-	Header text.Format // header row(s) text format
-	Row    text.Format // (data) row(s) text format
+	Direction text.Direction // (forced) BiDi direction for each Column
+	Footer    text.Format    // footer row(s) text format
+	Header    text.Format    // header row(s) text format
+	Row       text.Format    // (data) row(s) text format
 }
 
 var (
